import copy
import math
from typing import List, Tuple, Union
Seq = Union[List, Tuple]
try:
    from wcwidth import wcswidth
except ImportError:
    print("[text table] 请先安装wcwidth库: pip install wcwidth")

class MyDict:
    def __init__(self, init_dict:dict|None = None):
        self.dict = init_dict if init_dict is not None else dict()

    def update(self, other_dict):
        self.dict.update(other_dict)

    def items(self):
        return self.dict.items()
    def keys(self):
        return self.dict.keys()
    def values(self):
        return self.dict.values()

    def __getitem__(self, item):
        return self.dict[str(item)]

    def __setitem__(self, key, value):
        self.dict[str(key)] = value

class B_XYTable:
    def __init__(self, x_name, y_name, x_sidebars=[], y_sidebars=[]):
        self.x_sidebars = self._seq2strlist(x_sidebars)
        self.y_sidebars = self._seq2strlist(y_sidebars)
        self.x_name = x_name
        self.y_name = y_name
        self.dict = self._create_dict(self.x_sidebars, self.y_sidebars)
        self._widths = {x: 0 for x in self.y_sidebars}
        self._update_widths(MyDict({y: y for y in self.y_sidebars}))

    def set(self, x_sidebar, y_sidebar, content):
        x_sidebar, y_sidebar, content = str(x_sidebar), str(y_sidebar), str(content)
        assert x_sidebar in self.dict.keys(), "x_sidebar 与 不是指定好的属性"
        assert y_sidebar in self.dict[x_sidebar].keys(), "y_sidebar 与 不是指定好的属性"
        self.dict[x_sidebar][y_sidebar] = str(content)

    def get(self, x_sidebar, y_sidebar):
        return self.dict[x_sidebar][y_sidebar]
    def items(self):
        '''
        (key1, key2, value)
        '''
        result = [(x, y, self.dict[x][y]) for x in self.x_sidebars for y in self.y_sidebars]
        return result

    def get_table_by_strs(self) -> List[str]:
        results = self._create_prefix()

        for x in self.dict.values():
            self._update_widths(x)

        str_dash = ''
        str_head = ''
        for y in self.y_sidebars:
            pre_space, suf_space = self._get_prefix_suffix(y, self._widths[y], ' ')
            pre_dash, suf_dash = self._get_prefix_suffix('-', self._widths[y], '-')
            str_head += ' ' + pre_space + y + suf_space + ' |'
            str_dash += '-' + pre_dash + '-' + suf_dash + '-+'
        results[0] += str_dash
        results[1] += str_head
        results[2] += str_dash

        offset = 3
        for index, y_dicts in enumerate(self.dict.values()):
            for key, value in y_dicts.items():
                pre_space, suf_space = self._get_prefix_suffix(value, self._widths[key], ' ')
                str_content = ' ' + pre_space + value + suf_space + ' |'
                results[index+offset] += str_content

        results[-1] += str_dash

        return results

    def get_table_by_str(self) -> str:
        result = ""
        strs = self.get_table_by_strs()
        for x in strs[:-1]:
            result += x + '\n'
        result += strs[-1]

        return result
    def print_table(self):
        print(self.get_table_by_str())

    def _create_dict(self, seq1, seq2):
        result = dict()
        for x in seq1:
            temp = MyDict()
            for y in seq2:
                temp.update({y: ''})
            result.update({x: temp})

        return result

    def _create_prefix(self):
        '''
        得到
        +-------+
        | x \ y |
        +-------+
        |   1   |
        |   2   |
        |   3   |
        +-------+
        '''
        results = []

        title = self.x_name + " \ " + self.y_name
        n = self._get_maxlength_from_list(self.x_sidebars)
        length = max(n, self._get_width(title))

        pre_dash, suf_dash = self._get_prefix_suffix("-", length, '-')
        str_dash = "+-" + pre_dash + "-" + suf_dash + "-+"
        results.append(str_dash)

        pre_space, suf_space = self._get_prefix_suffix(title, length, ' ')
        str_index = "| " + pre_space + title + suf_space + " |"
        results.append(str_index)
        results.append(str_dash)

        for x in self.x_sidebars:
            pre_space, suf_space = self._get_prefix_suffix(x, length, ' ')
            str_number = "| " + pre_space + x + suf_space + " |"
            results.append(str_number)
        results.append(str_dash)

        return results

    def _get_prefix_suffix(self, string, length, charactor=' '):
        prefix = ''
        suffix = ''
        str_len = self._get_width(string)

        delta = length - str_len
        if delta < 0:
            assert "string的宽度比length宽"
        elif delta == 0:
            pass
        else:
            prefix = charactor * math.floor(delta / 2)
            suffix = charactor * math.ceil(delta / 2)

        return prefix, suffix

    def _get_maxlength_from_list(self, lst: List[str]) -> int:
        temp = [self._get_width(x) for x in lst]
        if len(temp) == 0:
            return 0
        else:
            return max(temp)

    def _update_widths(self, new_dict: MyDict):
        temps = copy.deepcopy(self._widths)
        for key, value in new_dict.items():
            temps[key] = self._get_width(value)

        if len(self._widths) == 0:
            self._widths = temps
        else:
            for key, value in temps.items():
                if value > self._widths[key]:
                    self._widths[key] = value

    def _seq2strlist(self, seq):
        seq = copy.deepcopy(list(seq))
        for i, x in enumerate(seq):
            seq[i] = str(x)

        return seq

    def _get_width(self, string):
        return wcswidth(string)

    def __len__(self):
        return len(self.x_sidebars) * len(self.y_sidebars)
    def __contains__(self, item):
        for x in self.x_sidebars:
            for y in self.y_sidebars:
                if str(item) == self.dict[x][y]:
                    return True
        return False
    def __str__(self):
        return self.get_table_by_str()

    def __getitem__(self, index):
        return self.dict[str(index)]
    def __setitem__(self, index, value):
        self.dict[index] = value
if __name__ == '__main__':
    my_table = B_XYTable("x", "y", [123, 234], [345, 456])

    my_table[123]['345'] = 'qwq'
    my_table['123']['456'] = '456'
    my_table.set(234, '345', "awwwa")
    my_table.set('234', '456', "123")

    print(my_table)
    print(my_table[123][345])
    for key1, key2, value in my_table.items():
        print(key1, key2, value)

    print('qwq' in my_table)



import copy
import math
import os
from pathlib import Path
from typing import List, Tuple, Union
Seq = Union[List, Tuple]
try:
    from wcwidth import wcswidth
except ImportError:
    raise ImportError("[table] 请先安装wcwidth库: pip install wcwidth")

def get_width(string):
    return wcswidth(string)

def get_maxwidth_2d(matrix):
    widths = [0 for _ in range(len(matrix[0]))]
    for row in matrix:
        for i, element in enumerate(row):
            if element is None:
                element = ' '
            widths[i] = max(widths[i], get_width(element))

    diff_matrix = [[0 for _ in range(len(matrix[0]))] for _ in range(len(matrix))]
    for i in range(len(matrix)):
        for j in range(len(matrix[0])):
            element = matrix[i][j]
            if element is None:
                element = ' '
            diff_matrix[i][j] = widths[j] - get_width(element)

    return widths, diff_matrix

def get_T_matrix(matrix):
    new_matrix = [[matrix[j][i] for j in range(len(matrix))] for i in range(len(matrix[0]))]
    return new_matrix

def strs2matrix(str_lst):
    str_lst = [x.strip() for x in str_lst if len(x.strip()) > 0]
    str_lst.pop(-1)
    str_lst.pop(2)
    dividing_line = str_lst.pop(0)
    indexes = [i for i, c in enumerate(dividing_line) if c == '+']
    row_cnt = len(str_lst)
    col_cnt = len(indexes) - 1


    matrix = []
    for i in range(row_cnt):
        row = str_lst[i].split('|')[1:-1]
        for j in range(col_cnt):
            row[j] = row[j].strip()
        matrix.append(row)

    row_name, col_name = matrix[0][0].split(' \\ ')

    return matrix, row_name, col_name


class Matrix2d:
    def __init__(self, row_name='x', col_name='y'):
        self.row_name = row_name
        self.col_name = col_name
        self.matrix = [[row_name + ' \\ ' + col_name]]

    def get_rows(self): # 包含left_upper
        if self.matrix == [[]]:
            return []
        return [x[0] for x in self.matrix]
    def get_cols(self): # 包含left_upper
        return self.matrix[0]
    def set_by_str(self, row, col, value):
        row, col, value = str(row), str(col), str(value)

        rows = self.get_rows()
        cols = self.get_cols()

        if row in rows:
            row_index = rows.index(row)
        else:
            row_index = len(rows)
        if col in cols:
            col_index = cols.index(col)
        else:
            col_index = len(cols)

        self.set_by_index(row_index, col_index, row, col, value)

    def set_by_index(self, row_index, col_index, row, col, value):
        row, col, value = str(row), str(col), str(value)

        rows = self.get_rows()
        cols = self.get_cols()

        if row_index < len(rows):
            pass
        elif row_index == len(rows):
            self.matrix.append([row] + [None] * (len(cols) - 1))

        if col_index < len(cols):
            pass
        elif col_index == len(cols):
            self.matrix[0].append(col)
            for i in range(1, len(self.matrix)):
                self.matrix[i].append(None)

        self.matrix[row_index][col_index] = value

    def get_by_str(self, row, col):
        row, col = str(row), str(col)

        rows = self.get_rows()
        cols = self.get_cols()
        if row in rows and col in cols:
            row_index = rows.index(row)
            col_index = cols.index(col)
            return self.get_by_index(row_index, col_index)
        else:
            raise ValueError(f"row {row} or col {col} not found in table")

    def get_by_index(self, row_index, col_index):
        rows = self.get_rows()
        cols = self.get_cols()
        if row_index < len(rows) and col_index < len(cols):
            return self.matrix[row_index][col_index]
        else:
            raise ValueError(f"row_index {row_index} or col_index {col_index} out of range")

    def T(self):
        new_matrix = get_T_matrix(self.matrix)
        new_left_upper = self.col_name + ' \\ ' + self.row_name
        new_matrix[0][0] = new_left_upper

        self.row_name, self.col_name = self.col_name, self.row_name

        self.matrix = new_matrix

        return self
    def __str__(self):
        result = ''
        for x in self.matrix:
            result += str(x) + '\n'
        return result


class Dict1d:
    def __init__(self, dictionary=None):
        self.dict = {}
        if dictionary:
            for key, value in dictionary.items():
                self[key] = value

    def __getitem__(self, key):
        key = str(key)
        try:
            return self.dict[key]
        except KeyError:
            return None
    def __setitem__(self, key, value): # 用于最底层赋值
        key, value = str(key), str(value)
        if value == 'None':
            value = ' '
        self.dict[key] = value
    def set(self, key, value): # 用于非底层赋值（防止把子dict变为str）
        key = str(key)
        self.dict[key] = value


    def keys(self):
        return self.dict.keys()

    def values(self):
        return self.dict.values()

    def items(self):
        return self.dict.items()

    def copy(self):
        return copy.deepcopy(self)

    def __str__(self):
        return str(self.dict)

    def __repr__(self):
        return str(self.dict)

class Dict2d:
    def __init__(self):
        self.dict = Dict1d()

    # 先实现setitem，再实现getitem
    def __getitem__(self, key):
        key = str(key)
        if key not in self.dict.keys():
            self[key] = Dict1d()
        return self.dict[key]
    def __setitem__(self, key, new_dict: Dict1d | dict):
        key = str(key)
        self.dict.set(key, Dict1d(new_dict))
    def __str__(self):
        return str(self.dict)

class B_Table2d:
    def __init__(self, row_name='x', col_name='y', auto_adaptive=False, narrow_better=True):
        '''

        :param row_name:
        :param col_name:
        :param auto_adaptive: 是否采取自动翻转的策略，来提升显示效果
        :param narrow_better: 在auto_adaptive=True的情况下，是否优先选择横向窄的表格，避免换行显示
        '''
        self.row_name = row_name
        self.col_name = col_name
        self.auto_adaptive = auto_adaptive
        self.narrow_better = narrow_better

        self.matrix2d = Matrix2d(row_name, col_name) # 用于输出
        self.dict2d = Dict2d() # 用于输入

        # 判断是否改变
        self.is_changed = False

    def set(self, row, col, content):
        row, col, content = str(row), str(col), str(content)
        self[row][col] = content

    def get_str(self, row, col):
        return self[row][col]
    def get_int(self, row, col):
        return int(self[row][col])
    def get_float(self, row, col):
        return float(self[row][col])
    def get_bool(self, row, col):
        '''
        如果是字符串"True"或"1"，返回True，否则返回False
        '''
        temp = self[row][col]
        if temp == "True" or temp == "1":
            return True
        else:
            return False

    def __getitem__(self, row):
        self.is_changed = True

        row = str(row)
        return self.dict2d[row]

    def __setitem__(self, row, new_dict):
        self.is_changed = True

        row = str(row)
        self.dict2d[row] = new_dict

    def from_txt(self, txt_path, create:bool=False, use_txt_name:bool=False):
        '''
        :param txt_path:
        :param create: 如果文件不存在，则创建
        :param use_txt_name: 默认使用init时的row_name和col_name
        :return:
        '''
        if not os.path.exists(txt_path):
            if create:
                self.to_txt(txt_path)
            else:
                raise FileNotFoundError(f"文件不存在: {txt_path}")

        with open(txt_path, 'r', encoding='utf-8') as f:
            lines = f.readlines()
        matrix, row_name, col_name = strs2matrix(lines)
        if {self.row_name, self.col_name} != {row_name, col_name}:
            raise ValueError(f"行头列头正反均不对应，请检查: [{self.row_name}, {self.col_name}] ❌ [{row_name}, {col_name}]")

        if use_txt_name:
            self.row_name = row_name
            self.col_name = col_name
            self.matrix2d = Matrix2d(self.row_name, self.col_name)
            self.matrix2d.matrix = matrix
        else:
            left_upper = self.row_name + ' \\ ' + self.col_name
            matrix[0][0] = left_upper
            if self.row_name == row_name and self.col_name == col_name:
                self.matrix2d = Matrix2d(self.row_name, self.col_name)
                self.matrix2d.matrix = matrix
            elif self.row_name == col_name and self.col_name == row_name:
                self.matrix2d = Matrix2d(self.col_name, self.row_name)
                self.matrix2d.matrix = get_T_matrix(matrix)

        self.__matrix2dict()

    def to_txt(self, txt_path):
        with open(txt_path, 'w', encoding='utf-8') as f:
            f.write(str(self))

    def to_strs(self):
        if self.is_changed:
            self.__dict2matrix()
        if self.__if_need_T():
            self.T()

        result = []

        # 计算宽度
        widths, diff_matrix = get_maxwidth_2d(self.matrix2d.matrix)

        for i in range(len(self.matrix2d.matrix)):
            elements = []
            for j in range(len(self.matrix2d.matrix[i])):
                element = self.matrix2d.matrix[i][j]
                if element is None:
                    element = ' '
                diff = diff_matrix[i][j]
                diff_left = diff // 2
                diff_right = diff - diff_left
                element = ' '*diff_left + str(element) + ' '*diff_right
                elements.append(element)
            row = ' | '.join(elements)
            row = '| ' + row + ' |'
            result.append(row)

        # 添加三行分隔线
        dividing_line = '+' + '+'.join(['-' * (widths[i] + 2) for i in range(len(widths))]) + '+'
        result.append(dividing_line)
        result.insert(1, dividing_line)
        result.insert(0, dividing_line)

        return result

    def to_str(self):
        return '\n'.join(self.to_strs())

    def to_ini(self, path):
        '''
        输出为ini格式
        :param path:
        :return:
        '''
        if self.is_changed:
            self.__dict2matrix()

        groups = self.matrix2d.get_rows()[1:]
        keys = self.matrix2d.get_cols()[1:]
        with open(path, 'w', encoding='utf-8') as f:
            for i, group in enumerate(groups):
                f.write(f"[{group}]\n")
                for j, key in enumerate(keys):
                    value = self.matrix2d.get_by_index(i+1, j+1)
                    f.write(f"{key} = {value}\n")
                f.write("\n")

    def from_ini(self, path):
        '''
        输入为ini格式
        :param path:
        :return:
        '''
        with open(path, 'r', encoding='utf-8') as f:
            lines = f.readlines()

        self.dict2d = Dict2d()

        group = None
        for line in lines:
            line = line.strip()
            if line.startswith('[') and line.endswith(']'):
                group = line[1:-1]
            elif '=' in line:
                key, value = line.split('=')
                key = key.strip()
                value = value.strip()
                if value == 'None':
                    value = None
                self[group][key] = value

        self.__dict2matrix()

    def __str__(self):
        return self.to_str()

    def __repr__(self):
        if self.is_changed:
            self.__dict2matrix()

        result = ''
        for x in self.matrix2d.matrix:
            result += str(x) + '\n'
        return result

    def T(self):
        if self.is_changed:
            self.__dict2matrix()
        self.matrix2d.T()
        self.row_name, self.col_name = self.col_name, self.row_name
        self.__matrix2dict()

        return self

    def __matrix2dict(self): # __
        keys1 = self.matrix2d.get_rows()[1:]
        keys2 = self.matrix2d.get_cols()[1:]
        self.dict2d = Dict2d()
        for i, key1 in enumerate(keys1):
            for j, key2 in enumerate(keys2):
                value = self.matrix2d.get_by_index(i+1, j+1)
                self.dict2d[key1][key2] = value

        self.is_changed = False

    def __dict2matrix(self): # __
        left_upper = self.row_name + ' \\ ' + self.col_name
        # 处理keys1和keys2
        keys1 = [left_upper] + list(self.dict2d.dict.keys())
        keys2 = [left_upper]

        temp = []
        for key1 in self.dict2d.dict.keys():
            for key2 in self.dict2d.dict[key1].keys():
                if key2 not in temp:
                    temp.append(key2)

        keys2.extend(temp)

        # 处理matrix框架
        matrix = [keys2]
        for x in keys1[1:]:
            matrix.append([x] + [None] * (len(keys2) - 1))

        # 构建matrix2d
        self.matrix2d = Matrix2d(self.row_name, self.col_name)
        self.matrix2d.matrix = matrix
        for row in self.dict2d.dict.keys():
            for col in self.dict2d.dict[row].keys():
                value = self.dict2d.dict[row][col]
                self.matrix2d.set_by_str(row, col, value)

        self.is_changed = False
    def __if_need_T(self):
        '''
        尽管T，但只是出来的str是T过的，并没有改变原来的矩阵，
        所以赋值还是原来的字符索引
        :return:
        '''
        if not self.auto_adaptive:
            return False

        now_matrix2d = self.matrix2d
        T_matrix2d = copy.deepcopy(now_matrix2d).T()

        now_widths, _ = get_maxwidth_2d(now_matrix2d.matrix)
        T_widths, _ = get_maxwidth_2d(T_matrix2d.matrix)

        now_width = sum(now_widths)
        T_width = sum(T_widths)

        if T_width < now_width and self.narrow_better:
            return True
        elif T_width > now_width and not self.narrow_better:
            return True
        else:
            return False


if __name__ == '__main__':
    a = B_Table2d(row_name='data', col_name='score', auto_adaptive=False)
    # a[0][1] = 'a3'
    # a[11][22] = '123'
    # a[11][1] = 'dawd'
    a.from_txt('./a.txt')
    print(a)
    # a.T()
    # print(a)





import os
from pathlib import Path
import subprocess
import time

from ..Bbasic import B_Color


def args_process(args: tuple) -> list:
    lst = []
    for x in args:
        if type(x) is str:
            lst.append(x)
        elif type(x) is list:
            lst.extend(x)
    return lst

def b_run_cmd(
        *args: str,
        show: bool = True,
):
    '''
    可传入多个字符串, 在cmd中运行
    :param args:
    :param show: 若show=True, 则会单开一个cmd, 在cmd中运行
    :return:
    '''
    command = ''
    for i in range(len(args)):
        if i == len(args) - 1:
            command += str(args[i])
            break
        command += str(args[i]) + ' && '
    if show:
        command = f'start cmd /K "{command}"'
    # print(command)
    subprocess.run(command, shell=True)

def b_run_python(
    *args: str|list[str],
    limit_time: int|float|None = None,
    log_path: Path|None = None
):
    '''
    可传入多个字符串, 在当前python环境下运行
    :param args: 以python开头, 用于运行.py文件
    :param show:
    :return:
    '''
    def run_log(content=''):
        if log_path is not None:
            parent = Path(log_path).parent
            os.makedirs(parent, exist_ok=True)
            with open(log_path, 'w', encoding='utf-8') as f:
                f.write("=====================\n")
                for string in str_lst:
                    f.write("\t" + string + '\n')
                f.write("=====================\n")
                f.write(content)

    str_lst = args_process(args)

    print(f"{B_Color.GREEN}=====================")
    print("BRunPython 将在3秒后开始:")
    for string in str_lst:
        print("\t" + string)
    print(f"====================={B_Color.RESET}")
    time.sleep(3)

    for string in str_lst:
        try:
            command_lst = string.split(' ')
            run_log("正在执行: " + string)
            start = time.time()
            result = subprocess.run(command_lst, timeout=limit_time)
            end = time.time()
            delta_t = end - start
            h_t = int(delta_t // 3600)
            m_t = int((delta_t % 3600) // 60)
            s_t = int(delta_t % 60)
            if result.returncode != 0: # 报错
                index = str_lst.index(string)
                str_lst[index] = string + f"\t[!!!Error!!!] [{h_t}h {m_t}m {s_t}s]"
            else:
                index = str_lst.index(string)
                str_lst[index] = string + f"\t[~Success~] [{h_t}h {m_t}m {s_t}s]"
        except subprocess.TimeoutExpired:
            print(f"程序运行超过 {limit_time} 秒，已被强制终止")
            h_t = int(limit_time // 3600)
            m_t = int((limit_time % 3600) // 60)
            s_t = int(limit_time % 60)
            index = str_lst.index(string)
            str_lst[index] = string + f"\t[!!!Time limit!!!] [{h_t}h {m_t}m {s_t}s]"

    print(f"{B_Color.GREEN}====================={B_Color.RESET}")
    print(f"{B_Color.GREEN}BRunPython 结束:{B_Color.RESET}")
    for string in str_lst:
        if 'Time limit' in string:
            print(f"\t{B_Color.YELLOW}" + string + f"{B_Color.RESET}")
        elif 'Error' in string:
            print(f"\t{B_Color.RED}" + string + f"{B_Color.RESET}")
        else:
            print(f"\t{B_Color.GREEN}" + string + f"{B_Color.RESET}")
    print(f"{B_Color.GREEN}====================={B_Color.RESET}")

    run_log('结束')



if __name__ == '__main__':
    b_run_cmd("echo hello", "echo world", "echo awa", show=True)
    # b_run_python(
    #     r"python E:\byzh_workingplace\byzh-rc-to-pypi\test1.py",
    #     r"python E:\byzh_workingplace\byzh-rc-to-pypi\test2.py",
    #     limit_time=3,
    # )


import shutil
import os
from pathlib import Path
from typing import Literal

def get_parent_dir(path: Literal['__file__']) -> Path:
    '''
    获取 该py文件 所在的文件夹
    :param path: __file__
    '''
    parent_dir = Path(path).parent
    return parent_dir

def get_cwd() -> Path:
    '''
    获取 当前工作目录current working directory
    '''
    return Path.cwd()


def makedirs(path):
    def is_dir(path):
        path = Path(path)

        # 存在
        if os.path.isdir(path):
            return True

        # 不存在
        name = path.name
        if '.' in name:
            return False
        return True

    def is_file(path):
        path = Path(path)

        # 存在
        if os.path.isfile(path):
            return True

        # 不存在
        name = path.name
        if '.' in name:
            return True
        return False

    path = Path(path)

    if is_dir(path):
        os.makedirs(path, exist_ok=True)
    if is_file(path):
        os.makedirs(path.parent, exist_ok=True)

def rm(path):
    if os.path.isdir(path):
        shutil.rmtree(path)
    if os.path.isfile(path):
        os.remove(path)


def get_dirpaths_in_dir(root_dir_path, exclude_dir=['__pycache__', '.git']):
    result = []
    for root, dirs, files in os.walk(root_dir_path):
        for i, dir in enumerate(dirs):
            if str(dir) in exclude_dir:
                dirs.pop(i)
        path = Path(root)
        result.append(path)

    result = result[1:]

    return result

def get_filepaths_in_dir(root_dir_path, exclude_name=[], exclude_suffix=['.pyc'], exclude_dir=['.git']):
    file_paths = []
    for root, dirs, files in os.walk(root_dir_path):
        for i, dir in enumerate(dirs):
            if str(dir) in exclude_dir:
                dirs.pop(i)
        for file in files:
            file_path = os.path.join(root, file)
            file_path = Path(file_path)
            if file_path.name in exclude_name or file_path.suffix in exclude_suffix:
                continue
            file_paths.append(file_path)
    return file_paths

if __name__ == '__main__':
    # print(get_dirpaths_in_dir(r'E:\byzh_workingplace\byzh-rc-to-pypi'))
    a = get_filepaths_in_dir(r'E:\byzh_workingplace\byzh-rc-to-pypi')
    print(a)


import sys
import time
import threading

from ..Bbasic import B_Color


class B_Tqdm:
    def __init__(
            self,
            total: int|None = None,
            prefix: str = 'Processing',
            suffix: str = '',
            length: int = 20,
            fill: str = '█',
    ):
        """
        类似tqdm的进度条
        :param total: 总数
        :param prefix: 前缀
        :param suffix: 后缀
        :param length: 进度条长度(字符), 默认为20个字符长度
        :param fill: 填充字符
        """
        super().__init__()
        self.total = total
        self.prefix = prefix
        self.suffix = suffix
        self.length = length
        self.fill = fill
        self.start_time = 0
        self.current = 0

        self._lock = threading.Lock()

    def _format_time(self, seconds):
        """将秒数转换为mm:ss格式"""
        minutes = int(seconds // 60)
        seconds = int(seconds % 60)
        return f'{minutes:02}:{seconds:02}'

    def update(self, step=1, color=B_Color.BLUE, prefix=None, suffix=None):
        with self._lock:
            if self.current == 0:
                self.start_time = time.time()
            if prefix is not None:
                self.prefix = prefix
            if suffix is not None:
                self.suffix = suffix

            # 更新进度
            self.current += step

            # 计算已用时间
            elapsed_time = time.time() - self.start_time
            elapsed_str = self._format_time(elapsed_time)
            # 预估剩余时间
            if self.total is not None:
                estimated_time = elapsed_time / self.current * (self.total - self.current) if self.current > 0 else 0
                estimated_str = self._format_time(estimated_time)
            # 计算每秒处理的项数
            speed = self.current / elapsed_time if elapsed_time > 0 else 0

            # 更新进度条
            if self.total is not None:
                filled_length = int(self.length * self.current // self.total)
                bar = self.fill * filled_length + '-' * (self.length - filled_length) * len(self.fill)

                sys.stdout.write(f'\r{color}{self.prefix} |{bar}|'
                                 f' {self.current}/{self.total} -> {elapsed_str}<{estimated_str} | {speed:.1f} it/s |'
                                 f' {self.suffix}{B_Color.RESET}')
            else:
                sys.stdout.write(f'\r{color}{self.prefix} | {self.current} iters -> {elapsed_str} | {speed:.1f} it/s |'
                                 f' {self.suffix}{B_Color.RESET}')
            sys.stdout.flush()

            # 补回车
            if self.current == self.total:
                sys.stdout.write('\n')
                sys.stdout.flush()



import copy
import math
import os
from pathlib import Path
from typing import List, Tuple, Union
Seq = Union[List, Tuple]
try:
    from wcwidth import wcswidth
except ImportError:
    raise ImportError("[table] 请先安装wcwidth库: pip install wcwidth")

class InnerList:
    def __init__(self):
        self.key_list = []
        self.value_list = []

    def items(self):
        return zip(self.key_list, self.value_list)
    def keys(self):
        return self.key_list
    def values(self):
        return self.value_list
    def copy(self):
        return copy.deepcopy(self)
    def __getitem__(self, item):
        item = str(item)
        if item not in self.key_list:
            self.key_list.append(item)
            self.value_list.append('')
        index = self.key_list.index(item)
        return self.value_list[index]

    def __setitem__(self, key, value):
        key, value = str(key), str(value)
        if key not in self.key_list:
            self.key_list.append(key)
            self.value_list.append(value)
        index = self.key_list.index(key)
        self.value_list[index] = value


class OuterList:
    def __init__(self):
        self.key_list = []
        self.value_list = []

    def items(self):
        return zip(self.key_list, self.value_list)
    def keys(self):
        return self.key_list
    def values(self):
        return self.value_list
    def __getitem__(self, item: str):
        item = str(item)
        if item not in self.key_list:
            self.key_list.append(item)
            self.value_list.append(InnerList())
        index = self.key_list.index(item)
        return self.value_list[index]

    def __setitem__(self, key, value):
        key, value = str(key), value
        if key not in self.key_list:
            self.key_list.append(key)
            self.value_list.append(value)
        self.value_list[self.key_list.index(key)] = value

class MyMatrix:
    def __init__(self):
        self.matrix = []
        self.fill = ' '


    def add(self, i, j, value):
        if i > len(self.matrix)-1:
            self.matrix.extend([[] for _ in range(i-len(self.matrix)+1)])
        if j > len(self.matrix[i])-1:
            self.matrix[i].extend([self.fill for _ in range(j-len(self.matrix[i])+1)])
        self.matrix[i][j] = value

    def update_matrix(self):
        max_length = max([len(x) for x in self.matrix])
        for i in range(len(self.matrix)):
            self.matrix[i].extend([self.fill for _ in range(max_length-len(self.matrix[i]))])

    def get_matrix(self):
        self.update_matrix()
        return self.matrix

class B_AutoTable:
    def __init__(self, row_name='x', col_name='y', auto_adaptive=False):
        self.row_sidebar = []
        self.col_sidebar = []
        self.row_name = row_name
        self.col_name = col_name
        self.lists = OuterList()
        self.verbose_lists = []
        self._widths = []

        self.auto_adaptive = auto_adaptive

    def set(self, row, col, content):
        row, col, content = str(row), str(col), str(content)
        self.lists[row][col] = content

    def get_str(self, row, col):
        return self[row][col]
    def get_int(self, row, col):
        return int(self[row][col])
    def get_float(self, row, col):
        return float(self[row][col])
    def get_bool(self, row, col):
        temp = self[row][col]
        if temp == "True" or temp == "1":
            return True
        else:
            return False
    def items(self):
        '''
        (key1, key2, value)
        '''
        self._update_all()
        result = [(row, col, self[row][col]) for row in self.row_sidebar for col in self.col_sidebar]
        return result
    def copy_row(self, new_row, old_row):
        new_row, old_row = str(new_row), str(old_row)
        self[new_row] = self[old_row]
    def read_txt(self, path:Path, row_name:None|str=None, col_name:None|str=None):
        with open(path, 'r') as f:
            lines = f.readlines()
            lines = [x.strip() for x in lines if x.startswith('|')]

        # 所有内容
        temp = []
        for string in lines:
            elements = string.split('|')[1:-1]
            elements = [x.strip() for x in elements]
            temp.append(elements)

        # 边角内容
        x_name, y_name = temp[0][0].split(' \\ ') if ('\\' in temp[0][0]) else ("x", "y")
        x_sidebar = [var[0] for var in temp[1:]]
        y_sidebar = temp[0][1:]

        if row_name == y_name and col_name == x_name: # 不按txt的样子读, 而是翻折后读
            self.row_sidebar = y_sidebar
            self.col_sidebar = x_sidebar
            self.row_name = y_name
            self.col_name = x_name
            self.lists = OuterList()
            self._widths = []
            for i in range(len(y_sidebar)):
                for j in range(len(x_sidebar)):
                    self[y_sidebar[i]][x_sidebar[j]] = temp[j + 1][i + 1]
        else:
            self.row_sidebar = x_sidebar
            self.col_sidebar = y_sidebar
            self.row_name = x_name
            self.col_name = y_name
            self.lists = OuterList()
            self._widths = []
            for i in range(len(x_sidebar)):
                for j in range(len(y_sidebar)):
                    self[x_sidebar[i]][y_sidebar[j]] = temp[i+1][j+1]

    def to_txt(self, path):
        '''
        将表格内容写入文件
        :param path:
        :return:
        '''
        dir = Path(path).resolve().parent
        os.makedirs(dir, exist_ok=True)
        with open(path, 'w') as f:
            f.write(self.to_str())

    def update_txt(self, path):
        '''
        更新表格内容\n
        如果文件不存在，则创建文件
        :param path:
        :return:
        '''
        # 是否存在该文件
        if not os.path.exists(path):
            self.to_txt(path)
        else:
            current_lists = self.lists
            self.read_txt(path, self.row_name, self.col_name)
            new_lists = self.lists
            self.lists = self._merge_lists(new_lists, current_lists)

            self.to_txt(path)



    def to_strs(self) -> List[str]:
        transpose = self._need_transpose()
        self._update_all(transpose)
        self._update_widths()
        results = self._create_prefix()

        str_dash = ''
        str_head = ''
        for index, y in enumerate(self.col_sidebar):
            pre_space, suf_space = self._get_prefix_suffix(y, self._widths[index], ' ')
            pre_dash, suf_dash = self._get_prefix_suffix('-', self._widths[index], '-')
            str_head += ' ' + pre_space + y + suf_space + ' |'
            str_dash += '-' + pre_dash + '-' + suf_dash + '-+'
        results[0] += str_dash
        results[1] += str_head
        results[2] += str_dash

        offset = 3
        for i, y_list in enumerate(self.verbose_lists):
            for j, value in enumerate(y_list):
                pre_space, suf_space = self._get_prefix_suffix(value, self._widths[j], ' ')
                str_content = ' ' + pre_space + value + suf_space + ' |'
                results[i+offset] += str_content

        results[-1] += str_dash

        return results

    def to_str(self) -> str:
        result = ""
        strs = self.to_strs()
        for x in strs[:-1]:
            result += x + '\n'
        result += strs[-1]

        return result

    def _merge_lists(self, lists_base, lists_new):
        result = OuterList()

        for row_key, inner_list in lists_base.items():
            for col_key, value in inner_list.items():
                result[row_key][col_key] = value
        for row_key, inner_list in lists_new.items():
            for col_key, value in inner_list.items():
                result[row_key][col_key] = value

        return result

    def _update_all(self, transpose=False):
        if not transpose:
            row_sidebar = []
            col_sidebar = []
            my_matrix = MyMatrix()
            for x_key, x_list in self.lists.items():
                row_sidebar.append(x_key)
                i = row_sidebar.index(x_key)

                for y_key, value in x_list.items():
                    if y_key not in col_sidebar:
                        col_sidebar.append(y_key)
                    j = col_sidebar.index(y_key)

                    my_matrix.add(i, j, value)

            self.row_sidebar = row_sidebar
            self.col_sidebar = col_sidebar
            self.verbose_lists = my_matrix.get_matrix()
            self.row_name, self.col_name = self.row_name, self.col_name
        else:
            row_sidebar = []
            col_sidebar = []
            my_matrix = MyMatrix()
            for x_key, x_list in self.lists.items():
                col_sidebar.append(x_key)
                i = col_sidebar.index(x_key)

                for y_key, value in x_list.items():
                    if y_key not in row_sidebar:
                        row_sidebar.append(y_key)
                    j = row_sidebar.index(y_key)

                    my_matrix.add(j, i, value)

            self.row_sidebar = row_sidebar
            self.col_sidebar = col_sidebar
            self.verbose_lists = my_matrix.get_matrix()
            self.row_name, self.col_name = self.col_name, self.row_name

    def _update_widths(self):
        '''
        请先调用_update_sidebar_and_vlist
        :return:
        '''
        temp = [self._get_width(x) for x in self.col_sidebar]

        for i in range(len(self.verbose_lists)):
            for j in range(len(self.verbose_lists[i])):
                temp[j] = max(temp[j], self._get_width(self.verbose_lists[i][j]))

        self._widths = temp

    def _create_prefix(self):
        '''
        先调用_update_sidebar_and_vlist

        得到
        +-------+
        | x \ y |
        +-------+
        |   1   |
        |   2   |
        |   3   |
        +-------+
        '''
        front, behind, row_sidebar = self.row_name, self.col_name, self.row_sidebar


        results = []

        title = front + " \ " + behind
        n = self._get_maxlength_from_list(row_sidebar)
        target_length = max(n, self._get_width(title))

        pre_dash, suf_dash = self._get_prefix_suffix("-", target_length, '-')
        str_dash = "+-" + pre_dash + "-" + suf_dash + "-+"
        results.append(str_dash)

        pre_space, suf_space = self._get_prefix_suffix(title, target_length, ' ')
        str_index = "| " + pre_space + title + suf_space + " |"
        results.append(str_index)
        results.append(str_dash)

        for x in row_sidebar:
            pre_space, suf_space = self._get_prefix_suffix(x, target_length, ' ')
            str_number = "| " + pre_space + x + suf_space + " |"
            results.append(str_number)
        results.append(str_dash)

        return results

    def _get_prefix_suffix(self, string, length, charactor=' '):
        prefix = ''
        suffix = ''
        str_len = self._get_width(string)

        delta = length - str_len
        if delta < 0:
            assert "string的宽度比length宽"
        elif delta == 0:
            pass
        else:
            prefix = charactor * math.floor(delta / 2)
            suffix = charactor * math.ceil(delta / 2)

        return prefix, suffix

    def _get_maxlength_from_list(self, lst: List[str]) -> int:
        temp = [self._get_width(x) for x in lst]
        if len(temp) == 0:
            return 0
        else:
            return max(temp)

    def _get_width(self, string):
        return wcswidth(string)

    def _need_transpose(self):
        if self.auto_adaptive:
            self._update_all()

            normal_widths = [self._get_width(x) for x in self.col_sidebar]
            for i in range(len(self.verbose_lists)):
                for j in range(len(self.verbose_lists[i])):
                    normal_widths[j] = max(normal_widths[j], self._get_width(self.verbose_lists[i][j]))

            transpose_widths = [self._get_width(x) for x in self.row_sidebar]
            for i in range(len(self.verbose_lists)):
                for j in range(len(self.verbose_lists[i])):
                    transpose_widths[i] = max(transpose_widths[i], self._get_width(self.verbose_lists[i][j]))

            if sum(normal_widths) > sum(transpose_widths):
                return True
            return False
        return False


    def __len__(self):
        return len(self.row_sidebar) * len(self.col_sidebar)
    def __contains__(self, item):
        item = str(item)
        for x in self.row_sidebar:
            for y in self.col_sidebar:
                if item == self[x][y]:
                    return True
        return False
    def __str__(self):
        return self.to_str()

    def __getitem__(self, index):
        index = str(index)
        return self.lists[index]
    def __setitem__(self, index, value:InnerList):
        index = str(index)
        self.lists[index] = value.copy()


# if __name__ == '__main__':
#     my_table = BAutoTable("x", "y", auto_adaptive=True)
#
#     my_table[1][3] = "w"
#
#     my_table[2][2] = "b"
#     my_table[1][5] = "a"
#     my_table[3][5] = "b"
#
#     print(my_table)

if __name__ == '__main__':
    my_table = B_AutoTable("x", "y", auto_adaptive=False)

    my_table[1][3] = "w"

    my_table[2][2] = "b"
    my_table[1][5] = "a"
    my_table["wq"]["dw"] = "adawd"
    my_table["wqdq"]["ddasfsaddw"] = "adawd"

    print(my_table)
    for x, y, value in my_table.items():
        print(x, y, value)



import os
import zipfile
from pathlib import Path

from ..Btqdm import B_Tqdm

def b_archive_zip(
    source_path,
    output_path:str|Path=None,
    exclude_dirs:list[str]=None,
    exclude_files:list[str]=None,
    exclude_exts:list[str]=None,
    while_list:list[str]=None,
    contain_empty_folder:bool=True,
):
    '''
    压缩文件夹，排除 指定文件夹and指定后缀文件

    name代表前缀, ext代表后缀(包括.)

    :param source_path:
    :param output_path: 如果不传入, 则默认为source_path同名同路径的zip文件
    :param exclude_dirs: ['__pycache__', '.git', '.idea']
    :param exclude_files: ['.gitignore', 'README.md']
    :param exclude_exts: ['.csv', '.npy', '.pt', '.pth']
    :return:
    '''
    output_path = source_path + '.zip' if output_path is None else output_path
    def get_lst(lst):
        return [] if lst is None else lst
    exclude_dirs = get_lst(exclude_dirs)
    exclude_files = get_lst(exclude_files)
    exclude_exts = get_lst(exclude_exts)
    while_list = get_lst(while_list)


    my_tqdm = B_Tqdm(prefix='Archive')

    with zipfile.ZipFile(output_path, 'w', zipfile.ZIP_DEFLATED) as zipf:
        # 压缩文件:
        if os.path.isfile(source_path):
            arcname = os.path.basename(source_path)
            zipf.write(source_path, arcname)
            my_tqdm.update(1)
            return

        # 压缩文件夹:
        if os.path.isdir(source_path):
            for root, dirs, files in os.walk(source_path):
                # 排除指定文件夹dirs
                dirs[:] = [d for d in dirs if d in while_list or d not in exclude_dirs]
                # 排除指定文件files
                files = [f for f in files if f in while_list or f not in exclude_files]
                # 排除指定后缀文件files
                files = [f for f in files if f in while_list or not any(f.endswith(ext) for ext in exclude_exts)]

                # 压缩文件:
                for file in files:
                    file_path = os.path.join(root, file)
                    arcname = os.path.relpath(file_path, source_path) # 相对于source_path的相对路径
                    zipf.write(file_path, arcname)

                    my_tqdm.update(1)

                # 若是空文件夹，则压缩文件夹:
                if contain_empty_folder and (len(dirs) == 0 and len(files) == 0):
                    arcname = os.path.relpath(root, source_path)
                    folder_path = root
                    zipf.write(folder_path, arcname)
                    my_tqdm.update(1)

if __name__ == '__main__':
    b_archive_zip(
        source_path=r'E:\byzh_workingplace\byzh-rc-to-pypi\mnist',
        output_path=r'E:\byzh_workingplace\byzh-rc-to-pypi\awaqwq.zip',
        exclude_dirs=['__pycache__', 'com'],
        exclude_exts=['.ppt']
    )


from setuptools import setup, find_packages
import byzh_core
setup(
    name='byzh_core',
    version=byzh_core.__version__,
    author="byzh_rc",
    description="byzh_core是byzh系列的核心库，包含了一些常用的工具函数和类。",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    license='MIT',
    packages=find_packages(),
    install_requires=[
        'wcwidth',
    ],
)



import copy
import math
from typing import List, Tuple, Union
Seq = Union[List, Tuple]
try:
    from wcwidth import wcswidth
except ImportError:
    print("[text table] 请先安装wcwidth库: pip install wcwidth")

class B_RowTable:
    def __init__(self, head: Seq):
        self.head = self._seq2strlist(head)
        self.rows = []
        self._widths = []
        self._update_widths(self.head)

    def append(self, sequence: Seq):
        assert len(sequence) == len(self.head), "添加的 序列元素个数 与 head元素个数 不一致"
        self.rows.append(self._seq2strlist(sequence))
    def insert(self, index, sequence: Seq):
        assert len(sequence) == len(self.head), "插入的 序列元素个数 与 head元素个数 不一致"
        self.rows.insert(index, self._seq2strlist(sequence))
    def remove(self, sequence: Seq):
        self.rows.remove(self._seq2strlist(sequence))
    def pop(self, index=-1):
        return self.rows.pop(index)

    def get_table_by_strs(self) -> List[str]:
        results = self._create_prefix()

        for x in self.rows:
            self._update_widths(x)

        str_dash = ''
        str_head = ''
        for i, x in enumerate(self.head):
            pre_space, suf_space = self._get_prefix_suffix(x, self._widths[i], ' ')
            pre_dash, suf_dash = self._get_prefix_suffix('-', self._widths[i], '-')
            str_head += ' ' + pre_space + x + suf_space + ' |'
            str_dash += '-' + pre_dash + '-' + suf_dash + '-+'
        results[0] += str_dash
        results[1] += str_head
        results[2] += str_dash

        offset = 3
        for i, row in enumerate(self.rows):
            for j, x in enumerate(row):
                pre_space, suf_space = self._get_prefix_suffix(x, self._widths[j], ' ')
                str_content = ' ' + pre_space + x + suf_space + ' |'
                results[i+offset] += str_content

        results[-1] += str_dash

        return results

    def get_table_by_str(self) -> str:
        result = ""
        strs = self.get_table_by_strs()
        for x in strs[:-1]:
            result += x + '\n'
        result += strs[-1]

        return result

    def print_table(self):
        print(self.get_table_by_str())

    def _create_prefix(self):
        '''
        得到
        +-----+
        | num |
        +-----+
        |  1  |
        |  2  |
        |  3  |
        +-----+
        '''
        results = []
        # 编号的位数
        n = self._get_width(str(len(self.rows)))
        length = max(n, self._get_width("index"))

        pre_dash, suf_dash = self._get_prefix_suffix("-", length, '-')
        str_dash = "+-" + pre_dash + "-" + suf_dash + "-+"
        results.append(str_dash)

        pre_space, suf_space = self._get_prefix_suffix("index", length, ' ')
        str_index = "| " + pre_space + "index" + suf_space + " |"
        results.append(str_index)
        results.append(str_dash)

        for i in range(len(self.rows)):
            number = str(i)
            pre_space, suf_space = self._get_prefix_suffix(number, length, ' ')
            str_number = "| " + pre_space + number + suf_space + " |"
            results.append(str_number)
        results.append(str_dash)

        return results

    def _get_prefix_suffix(self, string, length, charactor=' '):
        prefix = ''
        suffix = ''
        str_len = self._get_width(string)

        delta = length - str_len
        if delta < 0:
            assert "string的宽度比length宽"
        elif delta == 0:
            pass
        else:
            prefix = charactor * math.floor(delta / 2)
            suffix = charactor * math.ceil(delta / 2)

        return prefix, suffix

    def _update_widths(self, seq):
        temps = []
        for x in seq:
            temps.append(self._get_width(x))

        if len(self._widths) == 0:
            self._widths = temps
        else:
            for i, x in enumerate(temps):
                if x > self._widths[i]:
                    self._widths[i] = x

    def _seq2strlist(self, seq):
        seq = copy.deepcopy(list(seq))
        for i, x in enumerate(seq):
            seq[i] = str(x)

        return seq

    def _get_width(self, string):
        return wcswidth(string)

    def __len__(self):
        return len(self.rows)
    def __str__(self):
        return self.get_table_by_str()
    def __getitem__(self, index):
        return self.rows[index]
    def __setitem__(self, index, value):
        self.rows[index] = self._seq2strlist(value)
    def __contains__(self, item):
        for row in self.rows:
            for x in row:
                if str(item) == x:
                    return True
        return False

if __name__ == '__main__':
    my_table = B_RowTable(['云编号', '名称', 'IP地址', 'aaaabbbbcccc'])
    my_table.append(["server01", "服务器01", "172.16.0.1", 'aacc'])
    my_table.append(["server02", "服务器02", "172.16.0.2", 'aacc'])
    my_table.append(["server03", "服务器03", "172.16.0.3", 'aacc'])
    my_table.append(["server04", "服务器04", "172.16.0.4", 'aacc'])
    my_table.append(["server05", "服务器05", "172.16.0.5", 'aacc'])
    my_table.append(["server06", "服务器06", "172.16.0.6", 'aacc'])
    my_table.append(["server07", "服务器07", "172.16.0.7", 'aacc'])
    my_table.append(["server08", "服务器08", "172.16.0.8", 'aacc'])
    my_table.append(["server09", "服务器09", "172.16.0.9", 'aacc'])
    my_table.append(["server10", "服务器10", "172.16.0.0", 'aacc'])
    my_table.append(["server11", "服务器11", "172.16.0.1", 'aacc'])

    my_table.remove(["server08", "服务器08", "172.16.0.8", 'aacc'])

    my_table[4] = ["server99", "服务器99", "172.16.0.9", '99999']
    print(my_table)
    print(len(my_table))

    print("服务器07" in my_table)